#************************************************************ 
#
# Component      CNF  
#
# Synopsis       Operating System and Packages
#
#
# Copyright 2001-2004 Sun Microsystems, Inc. All rights reserved.
#
#
#************************************************************
#
# #ident "@(#)OS.pm 1.17 04/01/26 SMI"
#
#************************************************************

#=head1 Operating System and Packages
#
#    This section also contains descriptions for error than can be detected
#    in various circumstances (generic checking).
#
#=over 4
#
#=cut

package OS ;

use VAR;
use LOG;

my $G_category = "OS" ;

#
#  constant to be checked
#

my $G_S8_release = "Solaris 8 2/02 s28s_u7wos_08a SPARC";
my $G_S9_release = "Solaris 9 s9_58shwpl3 SPARC";
my $G_OS_release = "";

#
#  Packages list
#
#  0: name of the package
#  1: local (LOCAL), shared (SHARED)
#  2: version or list of versions (when EXACT is tested) separated by a comma
#  3: on master-eligible (YES or NO)
#  4: on diskless (YES or NO)
#  5: on dataless (YES or NO)
#  6 check exactly one of the listed version (EXACT) or at least the specific version (AT_LEAST)
#  7: mandatory package
#  8: feature which the package is part of

my $G_package_name      = 0 ;
my $G_package_location  = 1 ;
my $G_package_version   = 2 ;
my $G_package_men       = 3 ;
my $G_package_diskless  = 4 ;
my $G_package_dataless  = 5 ;
my $G_package_comp      = 6 ;
my $G_package_mandatory = 7 ;
my $G_package_feature   = 8 ;

my @G_packages_S8 = (
		  # CGTP packages
		  ["SUNWnhtp8", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "CGTP" ] ,
		  ["SUNWnhtu8", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "CGTP" ] ,
		 ) ;

my @G_packages_S9 = (
		  # CGTP packages
		  ["SUNWnhtp9", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "CGTP" ] ,
		  ["SUNWnhtu9", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "CGTP" ] ,
		    ) ;

my @G_packages_common = (	 
		  # ADM packages
		  ["SUNWnhadm", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "ADM" ] ,

		  # CMM packages
		  ["SUNWnhcdt", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "BASE" ] ,
		  ["SUNWnhcmd", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "CMM" ] ,
		  ["SUNWnhcma", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "CMM" ] ,
		  ["SUNWnhcmb", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "CMM" ] ,

		  # SNDR packages
		  ["SUNWscmr",  "LOCAL",       "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "YES", "RNFS" ] ,
		  ["SUNWscmu",  "LOCAL",       "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "YES", "RNFS" ] ,
		  ["SUNWspsvr", "LOCAL",       "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "YES", "RNFS" ] ,
		  ["SUNWspsvu", "LOCAL",       "2.0",    "YES", "NO",  "NO",  "AT_LEAST", "YES", "RNFS" ] ,
		  ["SUNWrdcr",  "LOCAL",       "2.0",    "YES", "NO",  "NO",  "AT_LEAST", "YES", "RNFS" ] ,
		  ["SUNWrdcu",  "LOCAL",       "2.0",    "YES", "NO",  "NO",  "AT_LEAST", "YES", "RNFS" ] ,

		  # RNFS packages
		  ["SUNWnhfsa", "LOCAL",       "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "YES", "RNFS" ] ,
		  ["SUNWnhfsb", "LOCAL",       "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "YES", "RNFS" ] ,

		  # PMD packages
		  ["SUNWnhpma", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "PMD" ] ,
		  ["SUNWnhpmb", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "PMD" ] ,
		  ["SUNWnhpmn", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "PMD" ] ,
		  ["SUNWnhpms", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "PMD" ] ,

		  # NSM packages
		  ["SUNWnhnsa", "LOCAL",       "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "NO",  "NSM" ] ,
		  ["SUNWnhnsb", "LOCAL",       "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "NO",  "NSM" ] ,

		  # NLS packages
#		  ["SUNWnhnla", "LOCAL",       "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "NO",  "NLS" ] ,
#		  ["SUNWnhnlb", "LOCAL",       "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "NO",  "NLS" ] ,

		  # JDK packages
		  ["SUNWj3rt",  "LOCAL",       "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "YES", "JDK" ] ,

		  # MA and JDMK packages
		  ["SUNWnhmas", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "NMA" ] ,
		  ["SUNWjsnmp", "LOCAL",       "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "YES", "NMA" ] ,
		  ["SUNWjdrt",  "SHARED",      "4.2",    "YES", "NO",  "NO",  "AT_LEAST", "YES", "NMA" ] ,
		  ["SUNWnhmaj", "SHARED",      "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "YES", "NMA" ] ,
		  ["SUNWnhmal", "SHARED",      "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "YES", "NMA" ] ,
		  ["SUNWnhmad", "SHARED",      "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "YES", "NMA" ] ,

		  # RBS packages
		  ["SUNWnhrbb", "LOCAL",       "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "NO",  "RBS" ] ,
		  ["SUNWnhrbs", "LOCAL",       "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "NO",  "RBS" ] ,

		  # SMCT packages
		  ["SUNWnhsms", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "NO",  "SMCT" ] ,
			);

# current table
my @G_packages;

#
#  Patch list
#
#  0: name of the patch
#  1: local (LOCAL), shared (SHARED)
#  2: on master-eligible (YES or NO)
#  3: on diskless (YES or NO)

my $G_patch_name     = 0 ;
my $G_patch_location = 1 ;
my $G_patch_men      = 2 ;
my $G_patch_diskless = 3 ;
my $G_patch_dataless = 4;

my @G_patches_S8 = (
		    # CGTP patch
		    ["112281-02", "LOCAL",  "YES", "YES", "YES" ] ,
		    # SNDR patch
		    ["116710-01", "LOCAL",  "YES", "NO",  "NO"  ] ,
		   ) ;

my @G_patches_S9 = (
		    # CGTP patch
		    ["112233-01", "LOCAL",  "YES", "YES", "YES" ] ,
		    ["112902-06", "LOCAL",  "YES", "YES", "YES" ] ,
		    ["112904-01", "LOCAL",  "YES", "YES", "YES" ] ,
		    ["112917-01", "LOCAL",  "YES", "NO" , "YES" ] ,
		    ["112918-01", "LOCAL",  "YES", "NO" , "YES" ] ,
		    ["112919-01", "LOCAL",  "YES", "NO" , "YES" ] ,
		    # SNDR patch
		    ["116710-01", "LOCAL",  "YES", "NO",  "NO"  ] ,
		   ) ;

# current table
my @G_patches;

#
# package acessibility
#
my $G_swdb_local_dir = "/SUNWcgha/local/export/services" ;
my $G_swdb_services = "/SUNWcgha/services" ;
my $G_swdb_mounting_point = "/SUNWcgha/swdb" ;
my $G_swdb_accessible = FALSE ;
my $G_swdb_remote_access = FALSE ;  
my $G_swdb_dir = "" ;
my $G_swdb_basedir = "" ;
my $G_swdb_path_confshare = "var/sadm/system/admin" ;
my $G_swdb_file_confshare = "INST_RELEASE" ;

#--------------------------------------------------------------
#
# set_os_env
#
#    determine the OS version and set all the related value
#
#--------------------------------------------------------------

sub set_os_env {

  chomp(my $release = qx/uname -r/);
  if ($release eq "5.8") {
    push @G_packages, @G_packages_S8;
    push @G_packages, @G_packages_common;
    @G_patches  = @G_patches_S8;
    $G_OS_release  = $G_S8_release;
  } else {
    push @G_packages, @G_packages_S9;
    push @G_packages, @G_packages_common;
    @G_patches  = @G_patches_S9;
    $G_OS_release  = $G_S9_release;
  }
}
#--------------------------------------------------------------
#
#  determine_swdb
#
#  shared package info database is
#  . only accessible from one node during installation
#    (when mounting points are not yet defined)
#  . accessible from all noeds after reboot (mounting point
#    has been defined on all nodes)
#
#--------------------------------------------------------------

sub determine_swdb
  {
    # is the mounting point defined ?
    $result = qx{/usr/sbin/mount | /usr/bin/grep $G_swdb_mounting_point} ;

    if ($result eq "") {
      # no mounting point: check if we are on the main node for installation
      if ( -d $G_swdb_local_dir ) {
	$G_swdb_accessible = TRUE ;
	$G_swdb_dir = $G_swdb_local_dir ;
      }

    } else {

      #
      # determine the base dir
      #
      $result = qx{/usr/sbin/mount | /usr/bin/grep $G_swdb_services};
      if ($result eq "") {
	LOG::printError("Can't determine the current BASE_DIR for shared packages") ;
	return ;
      }
      # extract the remote sub-directory name as the basedir
      if ($result !~ m{$G_swdb_local_dir/([^\s\t]*)}) {
 	LOG::printError("Can't determine the current BASE_DIR for shared packages") ;
	return ;
      }
      $G_swdb_basedir = "/$1";
      $G_swdb_accessible = TRUE ;
      $G_swdb_remote_access = TRUE ;
      $G_swdb_dir = $G_swdb_mounting_point ;
    }
	
  }

#--------------------------------------------------------------
#
#  guess_node_role
#
#    try to guess the node role by testing the mounting point
#    of the root partition since CMM can't be used
#    and nhfs.conf may be not already configured
#
#--------------------------------------------------------------

sub guess_node_role
  {
    chomp($result = qx{mount -v | /usr/bin/grep " on / type "}) ;
    $result =~ m/ on \/ type ([a-z]*) /;
    if ($1 eq "ufs")
      {
	my $version ;
	my $name ;
	my $basedir ;
	
	my $return = get_package("SUNWnhfsa", "LOCAL", 
				 \$version, \$name, \$basedir) ;
	if ($return eq "OK") {
	  return "DISKFULL" ;
	} else {
	  return "DATALESS"
	}
      }

    return "DISKLESS" ;
  }

#--------------------------------------------------------------
#
#  compare_version
#
#--------------------------------------------------------------

sub compare_version {

  my ($actual, $expected, $option) = @_ ;
 
  if ($option eq EXACT)
      {
	foreach $version (split(",",$expected))
	  {
	    if ($actual eq $version)
	      {
		return TRUE;
	      }
	  }
	return FALSE ;
      }
    else
      {
	if ($option eq "AT_LEAST")
	  {
	    @part_current = split('\.', $actual) ;
	    @part_expected = split('\.', $expected) ;
	    $atleast = TRUE ;
	    $end = FALSE ;
	    $exp_size = scalar(@part_expected) ;
	    $cur_size = scalar(@part_current) ;
	    my $i = 0 ;
	    while (($i < $exp_size) and ($end eq FALSE))
	      {
		if ($i < $cur_size)
		  {
		    if ($part_current[$i] > $part_expected[$i])
		      {
			$end = TRUE ;
		      }
		    else
		      {
			if ($part_current[$i] < $part_expected[$i])
			  {
			    $atleast = FALSE ;
			    $end = TRUE ;
			  }
		      }
                  }
		else
		  {
		    $atleast = FALSE ;
		    $end = TRUE ;
		  }
		$i++ ;
	      }
	      
	    if ($atleast eq FALSE)
	      {
		return LESS ;
	      }
	  }
      }
}

#--------------------------------------------------------------
#
#  get_package
#
#    get info about a specified package
#
#    1 : in: name of the package
#    2 : in: location
#
#    3 : out: version of the package
#    4 : out: complete name
#    5 : out: base directory
#
#    return :
#    . OK if the package exists
#    . NA if the package is not part of this node (Not Applicable)
#    . NOK if the package doesn't exist
#
#--------------------------------------------------------------

sub get_package
  {
    ($L_package, $L_location, $R_version, $R_name, $R_basedir) = @_ ;

    my $command = "" ;
    my $dir = "" ;

    # basedir may be not present in package information so it should be initialized
    $$R_basedir = "" ; 

    if ($L_location eq "SHARED") {
      $dir = "$G_swdb_dir" ;
      $command = "-R $G_swdb_dir" ;
    } elsif ($L_location ne "LOCAL") {
      LOG::printError("Invalid location (%s) for package %s",
		    $L_location, $L_package) ;
      exit 1;
    }

    $L_result = system("/usr/bin/pkginfo $command $L_package > /dev/null 2>&1");

    if ($L_result != 0)
      {
	return NOK ;
      }

    @L_output = qx{/usr/bin/pkginfo $command -l $L_package} ;
 
    foreach $L_line (@L_output)
      {
	chomp($L_line) ;
	if ($L_line =~ m/ERROR:/)
	  {
	    return NOK ;
	  }
	if ($L_line =~ m/VERSION:[\s\t]*([0-9.]*)/)
	  {
	    $$R_version = $1 ;
	  }
	if ($L_line =~ m/NAME:[\s\t]*(.*)/)
	  {
	    $$R_name = $1 ;
	  }
	if ($L_line =~ m/BASEDIR:[\s\t]*(.*)/)
	  {
	    $$R_basedir = $1 ;

	    # for a shared package:
	    # the base dir is checked only if the package is accessible
	    # via the mounting point (meaning that the base dir has been retrieved)
	    # -> if the base dir is different, the package is not the right one
	    #    we consider it as not installed
	    #
	    # if the access is local (before mounting point has been set)
	    # the base dir was not retrieved so don't check anything !
	    #
	    #
	    if (($L_location eq "SHARED") 
		&& ($G_swdb_remote_access eq TRUE) 
		&& ($$R_basedir ne $G_swdb_basedir)) {
	      return NOK ;
	    }
	  }
      }

    return OK ;
      
  }

#--------------------------------------------------------------
#
#  is_feature_installed
#
#    check if at least one package of a feature is installed
#    return TRUE or FALSE
#
#--------------------------------------------------------------

sub is_feature_installed {

  my ($feature) = @_;

  my $name;
  my $location;
  my $version;
  my $basedir;

  foreach my $package (@G_packages) {
    if (@$package[$G_package_feature] eq $feature) {
      $name = @$package[$G_package_name],
      $location = @$package[$G_package_location];
      if (get_package($name, $location, \$version, \$name, \$basedir) eq "OK") {
	return "TRUE";
      }
    }
  }

  return "FALSE";

}

#--------------------------------------------------------------
#
#  check_package
#
#    check if a package is installed and if its version
#    is correct
#
#    1 : name of the package
#    2 : location
#    3 : version to be tested (can be a list if type=EXACT
#    4 : type of comparison
#        EXACT   : must match the exact version
#        AT_LEAST: minimal version
#
#--------------------------------------------------------------

sub check_package
  {
    my ($L_package, $L_location, $L_version, $L_comp, $L_mandatory) = @_ ;

    my $L_status = "OK" ;
    my $L_current_version ;
    my $L_current_name ;
    my $L_basedir ;

    my $return = get_package($L_package, $L_location, 
			     \$L_current_version, \$L_current_name, \$L_basedir) ;
#    if ($return eq NA)
#      {
#	return ;
#      }

    if ($return ne OK) {
      if ($L_mandatory eq "YES") {
	

#=item package <name> not installed.
#
#    Please install the specified package.
#
#=cut

	LOG::printError("package %s not installed", $L_package) ;
	my $status = "FAILED";
	LOG::printResult($status, "Package %s", $L_package) ;
      } else {
	LOG::printWarning("package %s not installed", $L_package) ;
      }
      return ;
    }	

    $status = compare_version($L_current_version, $L_version, $L_comp) ;

    if ($status eq FALSE)
      {

#=item package <name>: version <current> instead of <expected>
#
#    Please install the correct version of the specified package.
#
#=cut

	LOG::printError("package %s: version %s instead of %s",
			$L_package, $L_current_version, $L_version) ;
	$L_status = FAILED ;
      }
    else
      {
	if ($status eq "LESS")
	  {

#=item package <name>: version <current> (at least <minimum> required)
#
#    Please install a correct version of the specified package.
#
#=cut

	    LOG::printError("package %s: version %s (at least %s required)", 
			    $L_package, $L_current_version, $L_version) ;
	    $L_status = FAILED ;
	  }
      }
    
    LOG::printResult($L_status, "Package %s", $L_package) ;
  }


#--------------------------------------------------------------
#
#  check_package_array
#
#    check if a array of package is installed
#
#    1: package array
#    2: MEN or NMEN
#
#--------------------------------------------------------------

sub check_package_array
  {
    my ($pkgarr, $role) = @_ ;
    
    my $field ;
    
    if ($role eq "MEN") {
      $field = $G_package_men ;
    } elsif ($role eq "DISKLESS") {
      $field = $G_package_diskless ;
    } else {
      $field = $G_package_dataless ;
    }
    
    #
    # check local packages if it makes sense
    #

    LOG::printMsg(0,"") ;
    LOG::printMsg(0,"Local packages") ;
    LOG::printMsg(0,"") ;
    foreach $pkg (@$pkgarr)
      {
	if ((@$pkg[$field] eq "YES") && (@$pkg[$G_package_location] eq "LOCAL"))
	  {
	    check_package(@$pkg[$G_package_name], 
			  @$pkg[$G_package_location], 
			  @$pkg[$G_package_version],
			  @$pkg[$G_package_comp],
			  @$pkg[$G_package_mandatory]);
	  }
      }

    if ($G_swdb_accessible eq FALSE) {
      LOG::printMsg(0,"") ;
      LOG::printMsg(0,"Shared packages currently not accessible") ;
      LOG::printMsg(0,"") ;
      return ;
    }

    #
    # check shared packages
    #
    LOG::printMsg(0,"") ;
    LOG::printMsg(0,"Shared packages (on %s)", $G_swdb_dir) ;
    LOG::printMsg(0,"") ;
    foreach $pkg (@$pkgarr)
      {
	if (@$pkg[$G_package_location] eq "SHARED")
	  {
	    check_package(@$pkg[$G_package_name], 
			  @$pkg[$G_package_location], 
			  @$pkg[$G_package_version],
			  @$pkg[$G_package_comp],
			  @$pkg[$G_package_mandatory]);
	  }
      }
  }

#--------------------------------------------------------------
#
#  get_patch
#
#    get info about a specified patch
#
#    1 : in: name of the package
#    2 : in: location
#
#    return TRUE if the patch exists
#
#--------------------------------------------------------------

sub get_patch
  {
    ($L_patch, $L_location) = @_ ;

    my $optcmd = "" ;

    if ($L_location eq "SHARED") {
      $optcmd = "-R $G_swdb_dir" ;
    } elsif ($L_location ne "LOCAL") {
      LOG::printMsg("Invalid location (%s) for patch %s",
		    $L_location,  $L_patch) ;
      exit 1;
    }

    $L_result = qx{/usr/sbin/patchadd $optcmd -p | grep $L_patch};

    if ($L_result eq "")
      {
	return FALSE ;
      }

    return TRUE ;
  }

#--------------------------------------------------------------
#
#  check_patch
#
#    check if a patch is installed and if its version
#    is correct
#
#    1 : name of the patch
#    2 : location
#    3 : to check or not
#
#--------------------------------------------------------------

sub check_patch
  {
    ($L_patch, $L_location, $L_tocheck) = @_ ;

    if ($L_tocheck ne "YES") {
      return ;
    }

    my $L_status = OK ;

    if (($L_location eq "SHARED") && ($G_swdb_accessible eq FALSE)) {
      return ;
    }

    if (get_patch($L_patch, $L_location) ne TRUE)
      {

#=item Patch <patch> not installed
#
#    The specified patch must be installed.
#
#=cut

	LOG::printError("Patch %s not installed", $L_patch) ;
	$L_status = FAILED;
      }
    LOG::printResult($L_status, "Patch %s", $L_patch) ;
  }

#--------------------------------------------------------------
#
#  check_file
#
#    1 : name of the file to check (exists, readable)
#
#--------------------------------------------------------------

sub check_file
  {
    ($filename) = @_ ;

    if (! -e $filename)
      {

#=item <file> does not exist
#
#    This may be caused by a wrong pathname.
#
#=cut

	LOG::printError("%s does not exist", $filename) ;
	return FALSE ;
     }

    if (! -r $filename)
      {

#=item can not read <file>: permission denied
#
#    Please check that the root account can access the specified file.
#
#=cut

        LOG::printError("can't read %s: permission denied", $filename) ;
	return FALSE ;
      }

    return TRUE ;
  }

#--------------------------------------------------------------
#
#  check_service
#
#    check if a service is defined in /etc/services
#
#    1 : name of the service
#    2 : protocol (tcp/udp)
#
#--------------------------------------------------------------

sub check_service
  {
    ($L_service,$L_protocol) = @_ ;

    my $L_status = OK ;
    my $protocol;
    my $wrong_protocol;
    my $save_protocol;
    my $found;

    LOG::printMsg(2,"Check service %s, protocol %s",
		  $L_service, $L_protocol) ;

    #
    # crfs service must be defined in /etc/services
    # a service can appear twice with different protocol
    #
    $L_status = OK ;
    if (check_file("/etc/services") eq TRUE)
      {
	open(FH,"</etc/services") or die "/etc/services: $!";
	$found = FALSE ;
	$wrong_protocol = FALSE ;
	while (<FH>)
	  {
	    chomp($line = $_) ;
	    $line =~ s/#.*// ;
	    if ($line =~ m/[\s\t]*([^\s\t]*)[\s\t]*([0-9]*)\/([a-z]*)/)
              {
	        $service = $1 ;
	        $port = $2 ;
	        $protocol = $3 ;
	        if ($service eq $L_service)
	          {
		    if ($protocol eq $L_protocol)
		      {
		        $found = TRUE ;
		        $wrong_protocol = FALSE ;
		      }
		    else
		      {
		        $wrong_protocol = TRUE ;
			$save_protocol = $protocol;
		      }
	          }
	      }
           }

	if ($found eq TRUE)
	  {
	    $L_STATUS = OK ;
	    LOG::printResult($L_status, "%s defined in /etc/services",
			    $L_service) ;
	  }
	else
	  {
	  if ($wrong_protocol eq TRUE)
	    {

#=item <service> service: <current> instead of <expected>
#
#    Please correct the protocol for the specified service
#    in /etc/services.
#
#=cut

	      LOG::printError ("%s service: %s instead of %s", $L_service, $save_protocol, $L_protocol);
	      $L_status = FAILED ;
	    }
	  else
	    {

#=item <service> service must be defined in /etc/services
#
#    Please add an entry for the specified service in /etc/services.
#    This may cause daemons using it to work incorrectly.
#
#=cut
 
	      LOG::printError("%s service must be defined in /etc/services", $L_service) ;
	      $L_status = FAILED ;
	    }
	}
      }

    close (FH) ;
  }

#--------------------------------------------------------------
#
#  check_installation
#
#      check information related to the Solaris OS
#
#--------------------------------------------------------------

sub check_installation
  {
    my $L_status = OK ;

    LOG::printMsg(0, "") ;
    LOG::printMsg(0, "OS and Software checking") ;
    LOG::printMsg(0, "") ;

    my $bits = qx{/usr/bin/isainfo -b} ;
    if ($bits != "64") {
#=item Invalid kernel mode: <bits>-bit instead of 64-bit mode
#
#    Please install the correct Solaris version.
#
#=cut

	LOG::printError("Invalid kernel mode: %s-bit instead of 64-bit mode",
			$bits) ;
	$L_status = FAILED;
      }
    LOG::printResult($L_status,"64-bit kernel mode" ) ;

    $L_status = OK ;
    my $release = qx{/usr/bin/grep "Solaris" /etc/release};
    $release =~ s/^[\s\t]*//;
    $release =~ s/[\s\t]*$//;
    chomp($release);

    LOG::printMsg(3, "Current release: %s, expected release: %s", 
		  $release, $G_OS_release) ;

    if ($G_OS_release ne $release) {
      
#=item release <current> instead of <expected>
#
#    Please install the correct Solaris version.
#
#=cut

      LOG::printError("release '%s' instead of '%s'",
		      $release, $G_OS_release) ;
      $L_status = FAILED;
    }
    LOG::printResult($L_status, "OS release") ;

    #
    #  check various packages
    #

    determine_swdb ;

    if (guess_node_role eq "DISKFULL") {
      check_package_array(\@G_packages, "MEN") ;
    } elsif (guess_node_role eq "DISKLESS") {
      check_package_array(\@G_packages, "DISKLESS") ;
    } else {
      check_package_array(\@G_packages, "DATALESS") ;
    }

    #
    #  check various patches
    #
    LOG::printMsg(0, "") ;
    LOG::printMsg(0, "Patches") ;
    LOG::printMsg(0, "") ;
    LOG::printMsg(0, "Can take a long time ...") ;
    LOG::printMsg(0, "") ;
    for ($i = 0 ; $i < scalar(@G_patches) ; $i++) {
      if (guess_node_role eq "DISKFULL") {
	check_patch($G_patches[$i][$G_patch_name], 
		    $G_patches[$i][$G_patch_location],
		    $G_patches[$i][$G_patch_men]) ;
      } elsif (guess_node_role eq "DISKLESS") {
	check_patch($G_patches[$i][$G_patch_name], 
		    $G_patches[$i][$G_patch_location],
		    $G_patches[$i][$G_patch_diskless]) ;
      } else {
	check_patch($G_patches[$i][$G_patch_name], 
		    $G_patches[$i][$G_patch_location],
		    $G_patches[$i][$G_patch_dataless]) ;
      }
    }

    #
    # check if shared file for patchabitlity support is present 
    # 
    if ($G_swdb_accessible eq TRUE) {
      $L_status = FAILED ;
      if (-f "$G_swdb_dir/$G_swdb_path_confshare/$G_swdb_file_confshare") {
	$L_status = OK ;
      } else {
	
#=File for patchability of shared packages not present.
#
#    Please use the "nhadm confshare" command to create this file.
#
#=cut

	LOG::printError("File for patchability of shared packages not present.") ;
      }
    LOG::printResult($L_status, "INST_RELEASE file for shared packages") ;
    }
  }

#--------------------------------------------------------------
#
#  display_package_array
# 
#      display information regarding package array
#
#--------------------------------------------------------------

sub display_package_array
  {
    my ($pkgarr, $role) = @_ ;

    my $field ;
    my $L_name;
    my $L_location ;
    my $L_version ;
    my $L_basedir ;

    if ($role eq "MEN") {
      $field = $G_package_men ;
    } elsif ($role eq "DISKLESS") {
      $field = $G_package_diskless ;
    } else {
      $field = $G_package_dataless ;
    }

    determine_swdb ;

    #
    # display local packages
    #
    LOG::printMsg(0,"") ;
    LOG::printMsg(0,"Local packages") ;
    LOG::printMsg(0,"") ;
    foreach $pkg (@$pkgarr)
      {
	if ((@$pkg[$field] eq "YES") && (@$pkg[$G_package_location] eq "LOCAL"))
         {
	   $L_name = @$pkg[$G_package_name];
	   $L_location = @$pkg[$G_package_location];
	   my $return = get_package($L_name, $L_location, \$L_version, \$L_name, \$L_basedir) ;
	   if ($return eq OK)
	     {
	       printLine("    package %s", $L_package) ;
	       printLine("        name:        %s", $L_name) ;
	       printLine("        version:     %s", $L_version) ;
	       if ($L_basedir ne "") {
		 printLine("        basedir:     %s", $L_basedir) ;
	       }
	     }
	   elsif ($return eq NOK)
	     {
	       printLine("    package %s -> not installed", $L_package) ;
	     }
	}
      }

    if ($G_swdb_accessible eq FALSE) {
      LOG::printMsg(0,"") ;
      LOG::printMsg(0,"Shared packages currently not accessible") ;
      LOG::printMsg(0,"") ;
      return ;
    }

    #
    # display shared packages
    #
    LOG::printMsg(0,"") ;
    LOG::printMsg(0,"Shared packages (on %s)", $G_swdb_dir) ;
    LOG::printMsg(0,"") ;
    foreach $pkg (@$pkgarr)
      {
	if (@$pkg[$G_package_location] eq "SHARED")
         {
	   $L_name = @$pkg[$G_package_name];
	   $L_location = @$pkg[$G_package_location];
	   my $return = get_package($L_name, $L_location, \$L_version, \$L_name, \$L_basedir) ;
	   if ($return eq OK)
	     {
	       printLine("    package %s", $L_package) ;
	       printLine("        name:        %s", $L_name) ;
	       printLine("        version:     %s", $L_version) ;
	       if ($L_basedir ne "") {
		 printLine("        basedir:     %s", $L_basedir) ;
	       }
	     }
	   elsif ($return eq NOK)
	     {
	       printLine("    package %s -> not installed", $L_package) ;
	     }
	}
      }
  }
#--------------------------------------------------------------
#
#  display
#
#      display information regarding OS
#
#--------------------------------------------------------------

sub display
  {

    printLine("") ;
    printLine("System information") ;
    printLine("") ;

    chomp(my $hardware  = qx{/usr/bin/uname -i}) ;

    printLine("    node name:  %s", $E_node_name) ;
    printLine("    Hardware:   %s", $hardware) ;
    printLine("    OS release: %s", $G_OS_release) ;

    if (guess_node_role eq "DISKFULL") {
      display_package_array(\@G_packages, "MEN") ;
    } elsif (guess_node_role eq "DISKLESS") {
      display_package_array(\@G_packages, "DISKLESS") ;
    } else {
      display_package_array(\@G_packages, "DATALESS") ;
    }
  }

#--------------------------------------------------------------
#
#  configure_shared
#
#--------------------------------------------------------------

sub configure_shared
  {
    my ($dir) = @_ ;

    if ($dir eq "") {
      #
      # the default directory is used if accessible
      #
      determine_swdb ;
      if ($G_swdb_accessible eq FALSE) {
	LOG::printError("Directory for shared packages not accessible") ;
	return ;
      }
      $dir = $G_swdb_dir ;
    }

    $result = qx{/usr/bin/mkdir -p $dir/$G_swdb_path_confshare 2>&1} ;
    if ( $? ne 0 ) {
      LOG::printMsg(0,"Error when executing mkdir") ;
      LOG::printMsg(0,$result) ;
      exit 1 ;
    }

    my $filename = "$dir/$G_swdb_path_confshare/$G_swdb_file_confshare";

    $result = qx{/usr/bin/cp /$G_swdb_path_confshare/$G_swdb_file_confshare $filename 2>&1} ;
    if ( $? ne 0 ) {
      LOG::printMsg(0,"Error when executing copy") ;
      LOG::printMsg(0,$result) ;
      exit 1 ;
    }

    LOG::printMsg(0, "File %s created", $filename) ;
  }

#--------------------------------------------------------------
#
#  main
#
#--------------------------------------------------------------

#=back
#
#=cut

{
}
